package com.capitalware.ufm;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.jdom.JDOMException;

import com.capitalware.ufm.action.handlers.ProcessActions;
import com.capitalware.ufm.action.objects.*;
import com.capitalware.ufm.actions.*;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.InternalMessageQueue;
import com.capitalware.ufm.handlers.RunJob;
import com.capitalware.ufm.UFM;
import com.capitalware.ufm.mq.MQTrigger;
import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.objects.UFMException;
import com.capitalware.ufm.other.GeneralInfo;
import com.capitalware.ufm.xml.UFMXml;

/**
 * This class will handle all Universal File Mover actions.
 *
 * @author Roger Lacroix, Capitalware Inc.
 * @return 0 for ok or 99 for failed.
 * @version 1.0.0
 * @license Apache 2 License
 */
public class UFM
{
   private static final String               UFM_XML_FILENAME = "ufm.xml";
   //
   public  static final String               WHO_AM_I = "UFM";
   public  static Logger                     logger = Logger.getLogger(WHO_AM_I);
   private static String                     xmlFilename = null;
   public  static String                     globalErrorText = null;
   public  static Hashtable<String,UFMObject> actionObjectValues = new Hashtable<String,UFMObject>();
   public  static String                     processFileLock = null;
   public  static boolean                    processLock = false;
   public  static boolean                    useFileLocking = true;
   public  static OnErrorObject              onError = null;
   public  static ArrayList<Thread>          runningThreads = new ArrayList<Thread>();

   /**
    * The constructor
    */
   public UFM()
   {
      super();
   }
   
   /**
    * Get the UFM XML Workflow filename.
    * @return xmlFilename
    */
   public static String getXmlFilename()
   {
      return UFM.xmlFilename;
   }

   /**
    * Set the UFM XML Workflow filename.
    * @param xmlFilename
    */
   public void setXmlFilename(String xmlFilename)
   {
      UFM.xmlFilename = xmlFilename;
   }
   
   /**
    * Add a running thread to the list.
    * @param t
    */
   public static void addThread(Thread t)
   {
      runningThreads.add(t);
   }
   
   
   /**
    * We are shutting down so stop all threads.
    */
   public static void stopAllThreads()
   {
      Thread t = null;
      
      for (int i=0; i < runningThreads.size(); i++)
      {
         t = runningThreads.get(i);
         t.interrupt();
      }
   }

   /**
    * @param args
    */
   public static void main(String[] args)
   {
      UFM      efm = new UFM();
      int      exitCode = 0;
      char     processType = CWConstants.ONE_PASS;
      boolean  windowsService = false;
      boolean  doWorkFlag = false;

      if (args.length > 3)
      {
         GeneralInfo.dumpHeader();
         UFM.logger.fatal("Invalid number of command-line parameters");
         efm.usage();
         exitCode = 99;
         System.exit(exitCode);
      }
      else if (args.length == 0)
      {
         xmlFilename = UFM_XML_FILENAME;  /* set a default value */
      }
      else if ("WindowsService".equals(args[0]))
      {
         windowsService = true;
         xmlFilename = efm.handleServiceStartup();
      }
      else
      {
         if (efm.isTrigger(args))
            processType = CWConstants.TRIGGERED;
         else
            xmlFilename = args[0];
      }

      /**
       * Check for process locking
       */
      if(args.length == 3)
      {
         processFileLock = args[1];
         File pfFile = new File(processFileLock);
         boolean invert = Boolean.parseBoolean(args[2].toLowerCase());

         if(pfFile.exists() && invert == true)
         {
            doWorkFlag = true;
         }
         else if(pfFile.exists() && invert == false)
         {
            doWorkFlag = false;
            UFM.logger.info(WHO_AM_I + " is now ending as another instance is running.");
         }
         else if(!pfFile.exists() && invert == true)
         {
            doWorkFlag = false;
            UFM.logger.info(WHO_AM_I + " is now ending a lock file does not exist and inverse locking is enabled.");
         }
         else
         {
            try 
            {
               boolean success = pfFile.createNewFile();
               if(success)
               {
                  UFM.logger.info("Process lock file " + processFileLock + " created");
                  processLock = true;
                  doWorkFlag = true;
               }
               else
               {
                  doWorkFlag = false;
                  UFM.logger.fatal("Cannot create file for process lock: " + processFileLock);
               }
            }
            catch (IOException e)
            {
               doWorkFlag = false;
               UFM.logger.fatal( "IOException: " + e.getLocalizedMessage());
               if (e.getCause() != null)
                  UFM.logger.fatal(e.getCause().toString());
            }
         }
      }
      else
      {
         doWorkFlag = true;
      }
      
      if(doWorkFlag)
      {
         Runtime.getRuntime().addShutdownHook(new ShutdownThread());
         exitCode = efm.doWork(processType, true);
      }

      UFM.logger.info(WHO_AM_I + " is now ending.");

      System.exit(windowsService?0:exitCode);
   }

   /**
    * We have an UFM XML file now it is time to do the work.
    * @param processType
    * @param headerFlag
    * @return exitCode
    */
   public int doWork(char processType, boolean headerFlag)
   {
      int                        exitCode = 0;
      Hashtable<String,String>   properties = null;
      UFMXml                     fmx = null;
      InternalMessageQueue       iq = null;

      try
      {
         fmx = new UFMXml();
         fmx.setLogger(logger);
         fmx.readXMLFile(xmlFilename);
         if (headerFlag)
            GeneralInfo.dumpHeader();
         UFM.logger.info(WHO_AM_I + " has processed " + xmlFilename);
         onError = fmx.getOnError();
         properties = fmx.getProperties();
         if (headerFlag)
            GeneralInfo.globalValues(fmx.getProperties());

         ProcessActions pa = new ProcessActions(fmx.getStatusObject());
         iq = pa.getInternalQueue();
         exitCode = pa.doWork(fmx.getActions(), properties);

         UFM.logger.info("");

         if (exitCode != 0)
         {
            UFM.logger.fatal("UFM Workflow XML file is abnormally terminating.");

//            handleOnError(properties, iq);
         }
      }
      catch (JDOMException e)
      {
         UFM.logger.fatal("JDOMException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());
         globalErrorText = e.getLocalizedMessage();
         handleOnError(properties, iq);
         exitCode = 99;
      }
      catch (FileNotFoundException e)
      {
         UFM.logger.fatal("FileNotFoundException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());
         globalErrorText = e.getLocalizedMessage();
         handleOnError(properties, iq);
         exitCode = 99;
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());
         globalErrorText = e.getLocalizedMessage();
         handleOnError(properties, iq);
         exitCode = 99;
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());
         UFM.globalErrorText = e.getLocalizedMessage();
         handleOnError(properties, iq);
         exitCode = 99;
      }

      UFM.stopAllThreads();
      
      return exitCode;
   }

   /**
    * On error - perform either Execute or SendEmail action or both
    * @param onError
    * @param properties
    */
   public static void handleOnError(Hashtable<String,String> properties, InternalMessageQueue iq)
   {
      try
      {
         if (onError != null)
         {
            if (onError.eo != null)
            {
               UFM.logger.fatal("OnError 'Execute Action' is now being performed.");
               RunJob rj = new RunJob(iq);
               rj.launch(onError.eo.xmlFile, globalErrorText, properties, "", "");
            }
            
            if (onError.seo != null)
            {
               UFM.logger.fatal("OnError 'SendEmail Action' is now being performed.");
   
               SendEmailAction sea = new SendEmailAction(onError.seo, iq);
               sea.doWork();
            }
         }
      }
      catch (JDOMException e)
      {
         UFM.logger.fatal("JDOMException: OnError: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());
      }
      catch (FileNotFoundException e)
      {
         UFM.logger.fatal("FileNotFoundException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: OnError: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());
      }
   }

   /**
    * Was UFM triggered?
    * @param args
    * @return boolean
    */
   private boolean isTrigger(String[] args)
   {
      boolean flag = false;

      if ( ( (args.length == 1) || (args.length == 2) ) &&
           (args[0].length() >= CWConstants.MQTMC_V1_LENGTH) &&
           (args[0].startsWith("TMC ")) )
      {
         try
         {
            MQTrigger mqt = new MQTrigger(args[0]);

            /* Are there any parameters in the Process's UserData? */
            if ( !("".equals(mqt.getUserData())) )
            {
               xmlFilename = mqt.getUserData();
               UFM.logger.info("UserData="+xmlFilename);
            }

            /* Are there any parameters in the Queue's TriggerData? */
            if ( !("".equals(mqt.getTriggerData())) )
            {
               xmlFilename = mqt.getTriggerData();
               UFM.logger.info("TriggerData="+xmlFilename);
            }
            flag = true;
         }
         catch (Exception e)
         {
            UFM.logger.fatal("Exception: "+ e.getLocalizedMessage());
            flag = false;
         }
      }
      else
         flag = false;

      return flag;
   }

   /**
    * Lookup the UFM Windows service's name in the property file
    * for the UFM XML file to be used during this innovation.
    * @return xmlFilename
    */
   private String handleServiceStartup()
   {
      String xmlFilename = UFM_XML_FILENAME; // set default value
      FileInputStream in = null;
      Properties props = new Properties();

      try
      {
         File ff = new File(CWConstants.SERVICE_FILE_NAME);
         ff.getAbsolutePath();
         UFM.logger.info(CWConstants.SERVICE_FILE_NAME+" absolute path is '" + ff.getAbsolutePath() + "'");
         
         in = new FileInputStream(ff);
         props.load(in);

         String serviceName = System.getProperty("exe4j.launchName");
         if (serviceName != null)
         {
            xmlFilename = props.getProperty(serviceName, UFM_XML_FILENAME);
            UFM.logger.info("Found '" + xmlFilename + "' for service name '" + serviceName+"'");
         }
         else
         {
            xmlFilename = UFM_XML_FILENAME; // set default value
            UFM.logger.info("Using default value of '" + xmlFilename + "'");
         }
      }
      catch (FileNotFoundException e)
      {
         UFM.logger.fatal("FileNotFoundException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());
         xmlFilename = UFM_XML_FILENAME; // set default value
         UFM.logger.info("Using default value of '" + xmlFilename + "'");
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());
         xmlFilename = UFM_XML_FILENAME; // set default value
         UFM.logger.info("Using default value of '" + xmlFilename + "'");
      }
      finally
      {
         try
         {
            if (in != null)
               in.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            if (e.getCause() != null)
               UFM.logger.fatal(e.getCause().toString());
         }
      }

      return xmlFilename;
   }

   /**
    * Display how to use this program to the end-user.
    */
   private void usage()
   {
      UFM.logger.fatal("Usage : " + WHO_AM_I + " efm_filename.xml");
      //
      System.err.println();
      System.err.println("Usage : " + WHO_AM_I + " efm_filename.xml");
      System.err.println();
   }

   /**
    * This class will handle the shutdown.
    *
    */
   private static class ShutdownThread extends Thread
   {
      /**
       * The constructor
       */
      public ShutdownThread()
      {
      }

      /**
       * Do the work.
       */
      public void run()
      {
         UFM.logger.debug("UFM: ShutdownThread.");
         
         try
         {
            if(processLock)
            {
               UFM.logger.info("UFM: Shutdown - handle process file lock.");
               IO.deleteFile(processFileLock);
            }
         }
         catch (UFMException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         }
      }
   }
}
