package com.capitalware.ufm.actions;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.HttpPutFileObject;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.Tokens;

/**
 * This class will handle the HttpPutFile action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class HttpPutFileAction extends UFMAction
{
   private static final String LINE_END = "\r\n";
   private static final String TWO_HYPHENS = "--";
   private static final String BOUNDARY = "***232404jkg4220957934FW**";

   private HttpPutFileObject hgfo;

   /**
    * The constructor
    * @param co
    */
   public HttpPutFileAction(HttpPutFileObject hgfo)
   {
      super(hgfo.actionName);
      this.hgfo = hgfo;
   }
   
   /**
    * Perform the HttpPutFile work.
    * @return exitCode
    */
   public int doWork()
   {
      int exitCode = 0;
      String srcPathAndFilename;
      HttpURLConnection conn = null;
      FileInputStream   fis = null;
      DataOutputStream  dos = null;
      DataInputStream   dis = null;

      int bytesRead, bytesAvailable, bufferSize;
      int maxBufferSize = 1 * 1024 * 1024;

      byte[] buffer;

      try
      {
         srcPathAndFilename = Tokens.replace(hgfo.actionName, hgfo.srcFilename, IO.getFileName(hgfo.srcFilename), 1, hgfo.paClass.actionProperties);

         fis = new FileInputStream(new File(srcPathAndFilename));

         // open a URL connection to the Servlet
         URL url = new URL(hgfo.url);
         
         // Open a HTTP connection to the URL
         conn = (HttpURLConnection) url.openConnection();

         // Allow Inputs
         conn.setDoInput(true);

         // Allow Outputs
         conn.setDoOutput(true);

         // Don't use a cached copy.
         conn.setUseCaches(false);

         // Use a post method.
         conn.setRequestMethod("POST");

         conn.setRequestProperty("Connection", "Keep-Alive");

         conn.setRequestProperty("Content-Type","multipart/form-data;boundary=" + BOUNDARY);

         dos = new DataOutputStream(conn.getOutputStream());

         dos.writeBytes(TWO_HYPHENS + BOUNDARY + LINE_END);
         dos.writeBytes("Content-Disposition: form-data; name=\""+hgfo.formDataName+"\";" + " filename=\""+ IO.getFileName(hgfo.srcFilename) + "\"" + LINE_END);
         dos.writeBytes(LINE_END);
         
         // create a buffer of maximum size
         bytesAvailable = fis.available();
         bufferSize = Math.min(bytesAvailable, maxBufferSize);
         buffer = new byte[bufferSize];

         // read file and write it into form...

         bytesRead = fis.read(buffer, 0, bufferSize);

         while (bytesRead > 0)
         {
            dos.write(buffer, 0, bufferSize);
            bytesAvailable = fis.available();
            bufferSize = Math.min(bytesAvailable, maxBufferSize);
            bytesRead = fis.read(buffer, 0, bufferSize);
         }

         // send multipart form data necessary after file data...
         dos.writeBytes(LINE_END);
         dos.writeBytes(TWO_HYPHENS + BOUNDARY + TWO_HYPHENS + LINE_END);
      }
      catch (MalformedURLException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         exitCode = 8;
      }

      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         exitCode = 8;
      }
      finally
      {
         try
         {
            if (dos != null)
               dos.flush();
         }
         catch (IOException e)
         {}
         
         try
         {
            if (dos != null)
               dos.close();
         }
         catch (IOException e)
         {}
         
         try
         {
            if (fis != null)
               fis.close();
         }
         catch (IOException e)
         {}
      }

      /*
       * Output server response.
       */
      try
      {
         dis = new DataInputStream(conn.getInputStream());
         String str;
         while ((str = dis.readLine()) != null)
         {
            UFM.logger.info("Server: " + str);
         }
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException2: " + e.getLocalizedMessage());
      }
      finally
      {
         try
         {
            if (dis != null)
               dis.close();
         }
         catch (IOException e)
         {}
      }
      
      if (conn != null)
         conn.disconnect();

      if (!hgfo.onErrorFail)
         exitCode = 0; /* reset back to zero */

      return exitCode;
   }
}
