package com.capitalware.ufm.actions;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import org.jdom.JDOMException;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.ScpObject;
import com.capitalware.ufm.objects.ConnectionObject;
import com.capitalware.ufm.scp.objects.*;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.InternalMessageQueue;
import com.capitalware.ufm.internal.messages.UpdateStatus;
import com.capitalware.ufm.xml.ScpXml;

/**
 * This class will handle the SCP action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class ScpAction extends UFMAction
{
   private     ScpObject      so;
   private     ScpXml         sxh;
   private     InternalMessageQueue   iq = null;
   private     String         fileSeparator = "/"; 

   /**
    * The constructor
    * @param sfo
    */
   public ScpAction(ScpObject sfo, InternalMessageQueue iq)
   {
      super(sfo.actionName);
      this.so = sfo;
      this.iq = iq;
   }

   /**
    * Perform the work.
    * @return exitCode
    */
   public int doWork()
   {
      int                     exitCode = 0;
      ConnectionObject        co = null;
      ArrayList<ScpCmdObject> cmdsAL = null;
      JSch                    jsch = new JSch();
      Session                 session = null;
      
      so.printValues();
      
      sxh = new ScpXml();
      try
      {
         sxh.readXMLFile(so.xmlFile);
         
         co = sxh.getScpConnection();
         cmdsAL = sxh.getScpCmds();

         /* Dump out the values */
         co.printValues("SCP");
         printValues(cmdsAL);

         /* Check/validate the values */
         co.checkObject("SCP");
         checkObject(cmdsAL);

         /* */
         java.util.Properties config = new java.util.Properties();
         config.put("StrictHostKeyChecking", "no");

         /* connect to the SCP server. */
         session = jsch.getSession(co.userID, co.hostname, co.port);
         
         session.setConfig(config);
         session.setPassword(co.password);
         session.connect();
         
         /* Process SCP commands. */
         exitCode = processScpCommands(session, cmdsAL);
      }
      catch (JDOMException e)
      {
         UFM.logger.fatal("JDOMException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      catch (JSchException e)
      {
         UFM.logger.fatal("JSchException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      finally
      {
         if (session != null)
         {
            session.disconnect();
            UFM.logger.info("Disconnected from " + co.hostname);
         }

         if (!so.onErrorFail)
            exitCode = 0; /* reset back to zero */
      }
      
      return exitCode;
   }
   
   
   /**
    * Print out SCP command objects, if any.
    * @param cmdsAL
    */
   private void printValues(ArrayList<ScpCmdObject> cmdsAL)
   {
      if (cmdsAL != null)
      {
         UFM.logger.info("     SCP commands to be processed:");
         for (int i=0; i < cmdsAL.size(); i++)
         {
            UFM.logger.info("       SCP Command ["+(i+1)+"]: " + ((ScpCmdObject)cmdsAL.get(i)).cmdName);
            ((ScpCmdObject)cmdsAL.get(i)).printValues();
         }
      }
      
   }
   
   /**
    * Issue checkObject for each SCP command object
    * @param cmdsAL
    */
   private void checkObject(ArrayList<ScpCmdObject> cmdsAL)
   {
      if (cmdsAL != null)
      {
         for (int i=0; i < cmdsAL.size(); i++)
         {
            ((ScpCmdObject)cmdsAL.get(i)).performTokenization(so.paClass.actionProperties);
            ((ScpCmdObject)cmdsAL.get(i)).checkObject();
         }
      }
   }
   
   /**
    * Loop through SCP commands and process them.
    * @param fc
    * @param cmdsAL
    * @return exitCode
    * @throws IOException
    * @throws JSchException 
    */
   private int processScpCommands(Session session, ArrayList<ScpCmdObject> cmdsAL) throws IOException, JSchException
   {
      int                     exitCode = 0;
      ScpCmdObject            SCPco = null;
      
      /**
       * Process SCP commands.
       */
      if (cmdsAL != null)
      {
         for (int i=0; i < cmdsAL.size(); i++)
         {
            SCPco = cmdsAL.get(i);
            if (SCPco instanceof CopyFromObject)
            {
               exitCode = processCopyFromCommand(session, (CopyFromObject)SCPco);
            }
            else if (SCPco instanceof CopyToObject)
            {
               exitCode = processCopyToCommand(session, (CopyToObject)SCPco);
            }
         }
      }
      
      return exitCode;
   }
   
   /**
    * Process the get command
    * @param session
    * @param cfo
    * @return exitCode
    * @throws IOException
    */
   private int processCopyFromCommand(Session session, CopyFromObject cfo) //throws IOException
   {
      SimpleDateFormat  dateTimeFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
      int               exitCode = 0;
      String            localFile= IO.apendFileToPath(cfo.localDir, (cfo.localFilename == null?cfo.remoteFilename:cfo.localFilename));
      String            remoteFile = IO.apendFileToPath(cfo.remoteDir, cfo.remoteFilename, fileSeparator);
      Channel           channel = null;
      FileOutputStream  fos = null;
      OutputStream      out = null;
      InputStream       in = null;

      String command = "scp -f " + remoteFile;
      
      try
      {
         channel = session.openChannel("exec");
         ((ChannelExec) channel).setCommand(command);
   
         // get I/O streams for remote scp
         out = channel.getOutputStream();
         in = channel.getInputStream();
   
         channel.connect();
   
         byte[] buf = new byte[1024];
   
         // send '\0'
         buf[0] = 0;
         out.write(buf, 0, 1);
         out.flush();
   
         while (true)
         {
            int c = checkAck(in);
            if (c != 'C')
            {
               break;
            }
   
            // read '0644 '
            in.read(buf, 0, 5);
   
            long filesize = 0L;
            while (true)
            {
               if (in.read(buf, 0, 1) < 0)
               {
                  // error
                  break;
               }
               if (buf[0] == ' ')
                  break;
               filesize = filesize * 10L + (long) (buf[0] - '0');
            }

            /*
             * Remove the file name from the input stream. 
             */
            for (int i = 0;; i++)
            {
               in.read(buf, i, 1);
               if (buf[i] == (byte) 0x0a)
               {
//                  String file = new String(buf, 0, i);
                  break;
               }
            }
   
//            System.out.println("filesize="+filesize+", file="+file);
   
            // send '\0'
            buf[0] = 0;
            out.write(buf, 0, 1);
            out.flush();
   
            // read a content of lfile
            fos = new FileOutputStream(localFile);
            int foo;
            while (true)
            {
               if (buf.length < filesize)
                  foo = buf.length;
               else
                  foo = (int) filesize;
               foo = in.read(buf, 0, foo);
               if (foo < 0)
               {
                  // error
                  break;
               }
               fos.write(buf, 0, foo);
               filesize -= foo;
               if (filesize == 0L)
                  break;
            }
            
            if (checkAck(in) != 0)
            {
               UFM.logger.fatal("processCopyFromCommand() checkAck(in) not zero.");
               UFM.globalErrorText = "processCopyFromCommand() checkAck(in) not zero.";
               exitCode = 8;
               break;
            }
            
            UFM.logger.info("Retrieved file from " + remoteFile + " and stored as " + localFile);
            cfo.fileCount++;

            if (iq != null)
               iq.addMessage(new UpdateStatus("0", dateTimeFormat.format(new Date())+", 0, "+so.actionName+", "+so.labelName+", SCP_Command=get "+remoteFile+"\n"));

            // send '\0'
            buf[0] = 0;
            out.write(buf, 0, 1);
            out.flush();
         }
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      catch (JSchException e)
      {
         UFM.logger.fatal("JSchException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      finally
      {
         try
         {
            if (in != null)
               in.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
            exitCode = 8;
         }
         
         try
         {
            if (out != null)
               out.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
            exitCode = 8;
         }
         
         try
         {
            if (fos != null)
               fos.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
            exitCode = 8;
         }
         
         if (channel != null)
            channel.disconnect();
      }

      return exitCode;
   }
   
   /**
    * Process the put command
    * @param session
    * @param cto
    * @return exitCode
    * @throws IOException
    */
   private int processCopyToCommand(Session session, CopyToObject cto) //throws IOException
   {
      SimpleDateFormat  dateTimeFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
      int               exitCode = 0;
      String            remoteFile = IO.apendFileToPath(cto.remoteDir, (cto.remoteFilename == null?cto.localFilename:cto.remoteFilename), fileSeparator); 
      String            localFile = IO.apendFileToPath(cto.localDir, cto.localFilename);
      FileInputStream   fis = null;
      Channel           channel = null; 
      OutputStream      out = null;
      InputStream       in = null;
      boolean           ptimestamp = true;

      // exec 'scp -t rfile' remotely
      String command = "scp " + (ptimestamp ? "-p" : "") + " -t " + remoteFile;
      
      try
      {
         channel = session.openChannel("exec");
         ((ChannelExec) channel).setCommand(command);

         // get I/O streams for remote scp
         out = channel.getOutputStream();
         in = channel.getInputStream();

         channel.connect();

         if (checkAck(in) != 0)
         {
            UFM.logger.fatal("processCopyToCommand() checkAck(in) not zero.");
            UFM.globalErrorText = "processCopyToCommand() checkAck(in) not zero.";
            exitCode = 8;
            return exitCode;
         }

         File _lfile = new File(localFile);

         if (ptimestamp)
         {
            command = "T " + (_lfile.lastModified() / 1000) + " 0";
            // The access time should be sent here,
            // but it is not accessible with JavaAPI ;-<
            command += (" " + (_lfile.lastModified() / 1000) + " 0\n");
            out.write(command.getBytes());
            out.flush();
            if (checkAck(in) != 0)
            {
               System.exit(0);
            }
         }

         // send "C0644 filesize filename", where filename should not include
         // '/'
         long filesize = _lfile.length();
         command = "C0644 " + filesize + " ";
         if (localFile.lastIndexOf(File.separator) > 0)
         {
            command += localFile.substring(localFile.lastIndexOf(File.separator) + 1);
         }
         else
         {
            command += localFile;
         }
         command += "\n";
         out.write(command.getBytes());
         out.flush();
         if (checkAck(in) != 0)
         {
            System.exit(0);
         }

         // send a content of lfile
         fis = new FileInputStream(localFile);
         byte[] buf = new byte[1024];
         while (true)
         {
            int len = fis.read(buf, 0, buf.length);
            if (len <= 0)
               break;
            out.write(buf, 0, len); // out.flush();
         }
         fis.close();
         fis = null;
         // send '\0'
         buf[0] = 0;
         out.write(buf, 0, 1);
         out.flush();
         
         if (checkAck(in) != 0)
         {
            UFM.logger.fatal("processCopyToCommand() checkAck(in) not zero.");
            UFM.globalErrorText = "processCopyToCommand() checkAck(in) not zero.";
            exitCode = 8;
            return exitCode;
         }
         
         UFM.logger.info("Stored file " + localFile + " to " + remoteFile);
         cto.fileCount++;

         if (iq != null)
            iq.addMessage(new UpdateStatus("0", dateTimeFormat.format(new Date())+", 0, "+so.actionName+", "+so.labelName+", SCP_Command=put "+remoteFile+"\n"));
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      catch (JSchException e)
      {
         UFM.logger.fatal("JSchException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      finally
      {
         try
         {
            if (in != null)
               in.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
            exitCode = 8;
         }
         
         try
         {
            if (out != null)
               out.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
            exitCode = 8;
         }
         
         try
         {
            if (fis != null)
               fis.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
            exitCode = 8;
         }
         
         if (channel != null)
            channel.disconnect();
      }
      
      return exitCode;
   }
   
   /**
    * 
    * @param in
    * @return
    * @throws IOException
    */
   private int checkAck(InputStream in) throws IOException
   {
      int b = in.read();
      // b may be 0 for success,
      // 1 for error,
      // 2 for fatal error,
      // -1
      if (b == 0)
         return b;
      if (b == -1)
         return b;

      if (b == 1 || b == 2)
      {
         StringBuffer sb = new StringBuffer();
         int c;
         do
         {
            c = in.read();
            sb.append((char) c);
         } while (c != '\n');
         
         if (b == 1)
         { // error
            UFM.logger.fatal("checkAck(): error: " + sb.toString());
         }
         if (b == 2)
         { // fatal error
            UFM.logger.fatal("checkAck(): fatal error: " + sb.toString());
         }
      }
      return b;
   }
}
