package com.capitalware.ufm.mq;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import org.jdom.JDOMException;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.MQSendObject;
import com.capitalware.ufm.handlers.AES;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.InternalMessageQueue;
import com.capitalware.ufm.handlers.RunJob;
import com.capitalware.ufm.handlers.StringUtilities;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.handlers.JZip;
import com.capitalware.ufm.internal.messages.UpdateStatus;
import com.capitalware.ufm.mq.objects.MQObject;
import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.objects.UFMException;

import com.ibm.mq.*;
import com.ibm.mq.constants.CMQC;

/**
 * This class will handle all MQ interactions for putting a file as a message onto a queue.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class MQPutFile extends MyMQ
{
   /**
    * The constructor
    */
   public MQPutFile(MQObject qm)
   {
   	super(qm);
   }

   /**
    * Do a MQPUT1: Open queue, put a message from a file to the queue and then close it.
    *
    * @param mqso
    * @param iq
    * @throws UFMException 
    * @throws BadPaddingException 
    * @throws IllegalBlockSizeException 
    * @throws InvalidAlgorithmParameterException 
    * @throws NoSuchPaddingException 
    * @throws NoSuchAlgorithmException 
    * @throws InvalidKeyException 
    */
   public int simplePut1(MQSendObject mqso, InternalMessageQueue iq) throws UFMException, InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException
   {
      int exitCode = 0;
      MQQueue outQ = null;
      MQDistributionList outDL = null;
      
      if (mqo.disListOfQNames != null)
      {
         UFM.logger.info("simplePut1: Attempting to open distribution list '"+mqo.disListOfQNames+"'");
         outDL = openOutDL(mqo.disListOfQNames, isSetContextRequired(mqso));
      }
      else
      {
         UFM.logger.info("simplePut1: Attempting to open queue '"+mqo.qObject+"'");
         outQ = openOutQ(mqo.qObject, isSetContextRequired(mqso));
      }
      
      try
      {
         if ( (outQ != null) || (outDL != null) )
            putMsg(mqso, outQ, outDL, iq);
         else
            exitCode = 99;
      }
      catch (JDOMException e)
      {
         UFM.logger.fatal("JDOMException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 99;
      }
      finally
      {
         if (outQ != null)
            closeQ(outQ);
         if (outDL != null)
            closeDL(outDL);
      }

      return exitCode;
   }

   /**
    * This method will get a directory listing of all files to be sent then call simplePut1 for each file.
    * @param mqso
    * @param iq
    * @throws UFMException 
    * @throws BadPaddingException 
    * @throws IllegalBlockSizeException 
    * @throws InvalidAlgorithmParameterException 
    * @throws NoSuchPaddingException 
    * @throws NoSuchAlgorithmException 
    * @throws InvalidKeyException 
    */
   public int putManyMsgs(MQSendObject mqso, InternalMessageQueue iq) throws UFMException, InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException
   {
      int exitCode = 0;
      MQQueue outQ = null;
      MQDistributionList outDL = null;
      
      if (mqo.disListOfQNames != null)
      {
         UFM.logger.info("putManyMsgs: Attempting to open distribution list '"+mqo.disListOfQNames+"'");
         outDL = openOutDL(mqo.disListOfQNames, isSetContextRequired(mqso));
      }
      else
      {
         UFM.logger.info("putManyMsgs: Attempting to open queue '"+mqo.qObject+"'");
         outQ = openOutQ(mqo.qObject, isSetContextRequired(mqso));
      }

      try
      {
         if (outQ != null)
         {
            File[] fileList = IO.getFilteredFileList(mqso.srcFilename);
   
            for (int i=0; i < fileList.length; i++)
            {
               mqso.srcFilename = fileList[i].toString();
               mqso.remoteFilename = fileList[i].getName();
   
               putMsg(mqso, outQ, outDL, iq);
            }
         }
         else
         {
            exitCode = 99;
         }
      }
      catch (JDOMException e)
      {
         UFM.logger.fatal("JDOMException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 99;
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 99;
      }
      finally
      {
         if (outQ != null)
            closeQ(outQ);
         if (outDL != null)
            closeDL(outDL);
      }

      return exitCode;
   }

   /**
    * Determine the file size then select the appropriate method to put the file.
    *
    * @param mqso
    * @param outQ
    * @param outDL
    * @throws JDOMException
    * @throws BadPaddingException 
    * @throws IllegalBlockSizeException 
    * @throws InvalidAlgorithmParameterException 
    * @throws NoSuchPaddingException 
    * @throws NoSuchAlgorithmException 
    * @throws InvalidKeyException 
    * @throws UFMException 
    */
   private void putMsg(MQSendObject mqso, MQQueue outQ, MQDistributionList outDL, InternalMessageQueue iq) throws JDOMException, InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException, UFMException
   {
      if ( (new File(mqso.srcFilename)).length() <= CWConstants.MAX_GROUP_MSG_SIZE)
         putMsgLess4MB(mqso, outQ, outDL, iq);
      else
         putMsgGreater4MB(mqso, outQ, outDL, iq);
   }

   /**
    * Do a MQPUT1: Open queue, put a message from a file to the queue and then close it.
    *
    * @param mqso
    * @param outQ
    * @param outDL
    * @throws JDOMException
    * @throws InvalidAlgorithmParameterException 
    * @throws NoSuchPaddingException 
    * @throws NoSuchAlgorithmException 
    * @throws InvalidKeyException 
    * @throws BadPaddingException 
    * @throws IllegalBlockSizeException 
    * @throws UFMException 
    */
   private synchronized void putMsgLess4MB(MQSendObject mqso, MQQueue outQ, MQDistributionList outDL, InternalMessageQueue iq) throws JDOMException, InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException, UFMException
   {
      SimpleDateFormat dateTimeFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
      MQPutMessageOptions pmo = new MQPutMessageOptions();
      pmo.options = CMQC.MQPMO_FAIL_IF_QUIESCING;
      UFM.logger.info("putMsgLess4MB: Attempting to send a file '"+mqso.srcFilename+"'");

      try
      {
         MQMessage sendMsg = new MQMessage();
         
         if (createUFMMessage(sendMsg, mqso))
            pmo.options = pmo.options + CMQC.MQPMO_SET_IDENTITY_CONTEXT;
         
         if ( (mqso.crypto != null) && (mqso.format == CWConstants.FORMAT_ZIP) ) 
         {
            AES aes = new AES(mqso.crypto.passPhrase, mqso.crypto.keySize);
            sendMsg.write(aes.encrypt(JZip.compressData(IO.readFile(mqso.srcFilename))));
         }
         else if (mqso.crypto != null) 
         {
            AES aes = new AES(mqso.crypto.passPhrase, mqso.crypto.keySize);
//            byte[] b = IO.readFile(so.srcFilename);
//            byte[] e = aes.encrypt(b);
//            sendMsg.write(e);
            
            sendMsg.write(aes.encrypt(IO.readFile(mqso.srcFilename)));
         }
         else if (mqso.format == CWConstants.FORMAT_ZIP)
            sendMsg.write(JZip.compressData(IO.readFile(mqso.srcFilename)));
         else
            sendMsg.write(IO.readFile(mqso.srcFilename));

         if (outDL != null)
         {
            outDL.put(sendMsg, pmo); // put the message to distribution list
            UFM.logger.info("Put file '" + mqso.srcFilename + "' as a message to distribution list: " + "'"+mqo.disListOfQNames+"'");
         } 
         else
         {
            outQ.put(sendMsg, pmo); // put the message on the queue.
            UFM.logger.info("Put file '" + mqso.srcFilename + "' as a message to queue: " + "'"+mqo.qObject+"'");
         }

         if (iq != null)
            iq.addMessage(new UpdateStatus("0", dateTimeFormat.format(new Date())+", 0, "+mqso.toStringForStatusUpdate("MQPut")));
         
         mqso.fileCount++;
         mqso.msgCount++;

         handlePostCommon(mqso, iq);
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + e.getLocalizedMessage() + " : QObject: " + mqo.qObject);
         UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         UFM.globalErrorText = e.getLocalizedMessage();
      }
      catch (IOException ioex)
      {
         UFM.logger.fatal("IOException: " + ioex.getLocalizedMessage());
         UFM.globalErrorText = ioex.getLocalizedMessage();
      }
   }

   /**
    * Do a MQPUT1: Open queue, put a message from a file to the queue and then close it.
    *
    * @param mqso
    * @param outQ
    * @param outDL
    * @throws JDOMException
    * @throws InvalidAlgorithmParameterException 
    * @throws NoSuchPaddingException 
    * @throws NoSuchAlgorithmException 
    * @throws InvalidKeyException 
    * @throws BadPaddingException 
    * @throws IllegalBlockSizeException 
    * @throws UFMException 
    */
   private synchronized void putMsgGreater4MB(MQSendObject mqso, MQQueue outQ, MQDistributionList outDL, InternalMessageQueue iq) throws JDOMException, InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException, UFMException
   {
      SimpleDateFormat dateTimeFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
      boolean processingOK = false;
      RandomAccessFile in = null;
      FileChannel fileChannel = null;
      FileLock fileLock = null;
      long fileSize = 0;
      MQPutMessageOptions pmo = new MQPutMessageOptions();
      pmo.options = CMQC.MQPMO_FAIL_IF_QUIESCING + CMQC.MQPMO_LOGICAL_ORDER + CMQC.MQPMO_SYNCPOINT;
      UFM.logger.info("putMsgGreater4MB: Attempting to send a file '"+mqso.srcFilename);

      /*
       * UFM will send the data in chucks of almost 4MB.
       */
      long fileLen = (new File(mqso.srcFilename)).length();
      long numberOfBlocks = fileLen / CWConstants.MAX_GROUP_MSG_SIZE;
      int leftOver = (int)(fileLen % CWConstants.MAX_GROUP_MSG_SIZE);

      byte[] byteData = new byte[CWConstants.MAX_GROUP_MSG_SIZE];

      try
      {
         MQMessage sendMsg = new MQMessage();
         
         if (createUFMMessage(sendMsg, mqso))
            pmo.options = pmo.options + CMQC.MQPMO_SET_IDENTITY_CONTEXT;

         sendMsg.messageFlags = CMQC.MQMF_MSG_IN_GROUP;
         
         in = new RandomAccessFile(mqso.srcFilename, "r");
         fileChannel = in.getChannel();
         fileLock = fileChannel.tryLock(0, fileLen, true); /* shared lock for reading */

         if (fileLock == null)
         {
            UFM.logger.fatal("putMsgGreater4MB cannot acquire lock for file: " + mqso.srcFilename);
            UFM.globalErrorText = "putMsgGreater4MB cannot acquire lock for file: " + mqso.srcFilename;
            throw new UFMException("putMsgGreater4MB cannot acquire lock for file: " + mqso.srcFilename);
         }
         
         for (int i=0; i < numberOfBlocks; i++)
         {
            if ( (i == (numberOfBlocks - 1)) && (leftOver == 0) )
               sendMsg.messageFlags = CMQC.MQMF_LAST_MSG_IN_GROUP; 
            else
               sendMsg.messageFlags = CMQC.MQMF_MSG_IN_GROUP;

            in.read(byteData);
            
            if ( (mqso.crypto != null) && (mqso.format == CWConstants.FORMAT_ZIP) )
            {
               AES aes = new AES(mqso.crypto.passPhrase, mqso.crypto.keySize);
               sendMsg.write(aes.encrypt(JZip.compressData(byteData)));
            }
            else if (mqso.crypto != null)
            {
               AES aes = new AES(mqso.crypto.passPhrase, mqso.crypto.keySize);
               sendMsg.write(aes.encrypt(byteData));
            }
            else if (mqso.format == CWConstants.FORMAT_ZIP)
               sendMsg.write(JZip.compressData(byteData));
            else
               sendMsg.write(byteData);
            
            fileSize += byteData.length;
            
            if (outDL != null)
               outDL.put(sendMsg, pmo); // put the message to distribution list
            else
               outQ.put(sendMsg, pmo); // put the message on the queue.
            
            mqso.msgCount++;
            
            UFM.logger.info("Putting message # "+(i+1)+" in group from file '" + mqso.srcFilename);
            sendMsg.clearMessage();
            sendMsg.messageId = CMQC.MQMI_NONE; // each message should have a unique MsgId.

            if (iq != null)
               iq.addMessage(new UpdateStatus("0", dateTimeFormat.format(new Date())+", 0, MessageNumber="+(i+1)+", "+mqso.toStringForStatusUpdate("MQPut")));
         }

         if (leftOver != 0) 
         {
            sendMsg.messageFlags = CMQC.MQMF_LAST_MSG_IN_GROUP; 
            byteData = new byte[(int)leftOver];
            in.read(byteData);
            
            if ( (mqso.crypto != null) && (mqso.format == CWConstants.FORMAT_ZIP) )
            {
               AES aes = new AES(mqso.crypto.passPhrase, mqso.crypto.keySize);
               sendMsg.write(aes.encrypt(JZip.compressData(byteData)));
            }
            else if (mqso.crypto != null)
            {
               AES aes = new AES(mqso.crypto.passPhrase, mqso.crypto.keySize);
               sendMsg.write(aes.encrypt(byteData));
            }
            else if (mqso.format == CWConstants.FORMAT_ZIP)
               sendMsg.write(JZip.compressData(byteData));
            else
               sendMsg.write(byteData);
            
            fileSize += byteData.length;
            
            if (outDL != null)
               outDL.put(sendMsg, pmo); // put the message to distribution list
            else
               outQ.put(sendMsg, pmo); // put the message on the queue.
            
            mqso.msgCount++;
            UFM.logger.info("Putting message # "+(numberOfBlocks+1)+" (last) in group from file '" + mqso.srcFilename);

            if (iq != null)
               iq.addMessage(new UpdateStatus("0", dateTimeFormat.format(new Date())+", 0, MessageNumber="+(numberOfBlocks+1)+", "+mqso.toStringForStatusUpdate("MQPut")));
         }
         
         mqso.fileCount++;
         
         UFM.logger.info("Put group of messages to queue "+mqo.qObject +" : Filesize is "+fileSize);
         
         _qMgr.commit();
         UFM.logger.info("Issued commit." );

         processingOK = true;
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + e.getLocalizedMessage() + " : QName=" + mqo.qObject);
         UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         UFM.globalErrorText = e.getLocalizedMessage();
         backout();
         processingOK = false;
      }
      catch (IOException ioex)
      {
         UFM.logger.fatal("IOException: " + ioex.getLocalizedMessage());
         UFM.globalErrorText = ioex.getLocalizedMessage();
         backout();
         processingOK = false;
      }
      finally
      {
         try
         {
            if(fileLock != null) 
               fileLock.release();         
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         }
         
         try
         {
            if (fileChannel != null)
               fileChannel.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         }
         
         try
         {
            if (in != null)
               in.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         }
      }
      
      try
      {
         if (processingOK)
            handlePostCommon(mqso, iq);
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
      }
   }

   /**
    * Create an MQMessage for UFM to use.
    *
    * @param sendMsg
    * @param mqso
    * @return flag
    * @throws JDOMException
    * @throws IOException
    * @throws MQException 
    */
   private synchronized boolean createUFMMessage(MQMessage sendMsg, MQSendObject mqso) throws JDOMException, IOException, MQException
   {
      boolean flag = false;
      String remoteJobFile = null;
      String remoteFilename = null;
      String remoteDirectory = null;
      UFM.logger.debug("Creating MQMessage for UFM.");

      sendMsg.messageId    = CMQC.MQMI_NONE;
      sendMsg.correlationId= CMQC.MQCI_NONE;
      sendMsg.groupId      = CMQC.MQGI_NONE;
      sendMsg.persistence  = (mqso.persistence? CMQC.MQPER_PERSISTENT : CMQC.MQPER_NOT_PERSISTENT);
      sendMsg.priority     = mqso.priority;
      sendMsg.characterSet = mqso.ccsid;
      sendMsg.messageType  = CMQC.MQMT_DATAGRAM;
      sendMsg.format       = CMQC.MQFMT_NONE;
      sendMsg.feedback     = CWConstants.UFM_MSG;

      if (mqo.userID != null)
         sendMsg.userId = mqo.userID;
      else
         sendMsg.userId = System.getProperty("user.name");

      if ( (mqso.crypto != null) && (mqso.format == CWConstants.FORMAT_ZIP) )
         sendMsg.feedback = CWConstants.UFM_ZIP_ENCRYPT_MSG;
      else if (mqso.crypto != null)
         sendMsg.feedback = CWConstants.UFM_ENCRYPT_MSG;
      else if (mqso.format == CWConstants.FORMAT_ZIP)
         sendMsg.feedback = CWConstants.UFM_ZIP_MSG;
      else if (mqso.format == CWConstants.FORMAT_STRING)
         sendMsg.format   = CMQC.MQFMT_STRING;
      
      if (mqso.remoteJobXmlFile != null)
         remoteJobFile = Tokens.replace(mqso.actionName, mqso.remoteJobXmlFile, mqso.remoteJobXmlFile);

      remoteFilename = Tokens.replace(mqso.actionName, mqso.remoteFilename, mqso.srcFilename);

      remoteDirectory = Tokens.replace(mqso.actionName,
                                       mqso.remoteDirectory,
                                       IO.getPath(mqso.srcFilename) );

      if (mqso.usev7prop)
      {
         sendMsg.feedback = sendMsg.feedback | CWConstants.UFM_V7_PROPERTIES_MSG;
         
         if (remoteJobFile != null)
            sendMsg.setStringProperty(CWConstants.UFM_PROP_EXECUTE, remoteJobFile);
         
         sendMsg.setStringProperty(CWConstants.UFM_PROP_DIRECTORY, remoteDirectory);
         sendMsg.setStringProperty(CWConstants.UFM_PROP_FILENAME, remoteFilename);
         flag = false;
      }
      else
      {
         /**
          * Set MQMD Context fields if the data fits or 
          * create XML and put the XML in the payload.
          */
         if ( (remoteFilename.length() > CMQC.MQ_APPL_IDENTITY_DATA_LENGTH)   ||
              (remoteDirectory.length() > CMQC.MQ_ACCOUNTING_TOKEN_LENGTH)    ||
              ( (remoteJobFile != null) &&
                ((remoteJobFile.length() - CWConstants.XML_SUFFIX.length()) > CMQC.MQ_CORREL_ID_LENGTH) ) )
         {
            sendMsg.feedback = sendMsg.feedback | CWConstants.UFM_XML_EMBEDDED_MSG;

            StringBuffer xmlData = new StringBuffer();
            xmlData.append(CWConstants.XML_PROLOG);
            xmlData.append("<" + CWConstants.UFM_MSG_ELEMENT + ">");
            xmlData.append("<" + CWConstants.REMOTE_ELEMENT + ">");
            if (remoteJobFile != null)
            {
               xmlData.append("<" + CWConstants.EXECUTE_ELEMENT + " " + CWConstants.XML_FILE_ATTRIBUTE + "=\""  + remoteJobFile + "\" />" );
               sendMsg.feedback = sendMsg.feedback | CWConstants.UFM_START_JOB;
            }
            xmlData.append("<" + CWConstants.DIRECTORY_ELEMENT + ">" + remoteDirectory +"</" + CWConstants.DIRECTORY_ELEMENT + ">");
            xmlData.append("<" + CWConstants.FILE_ELEMENT + ">" + remoteFilename + "</" + CWConstants.FILE_ELEMENT + ">");
            xmlData.append("</" + CWConstants.REMOTE_ELEMENT + ">");
            xmlData.append("</" + CWConstants.UFM_MSG_ELEMENT + ">");
            byte[] xmlByteData = xmlData.toString().getBytes();

            /* save the XML data length */
            int len = xmlByteData.length;
            sendMsg.correlationId[0] = (byte) ((len >>> 24) & 0xffL);
            sendMsg.correlationId[1] = (byte) ((len >>> 16) & 0xffL);
            sendMsg.correlationId[2] = (byte) ((len >>> 8)  & 0xffL);
            sendMsg.correlationId[3] = (byte) ((len) & 0xffL);

            UFM.logger.debug("xmlData.length()="+xmlData.length() + " : xmlByteData.length="+xmlByteData.length + " : len=" + len + " : CorrelID_HEX="+StringUtilities.byteArrayToHexString(sendMsg.correlationId));
            
            sendMsg.write(xmlByteData);
            flag = false;
         }
         else
         {
            sendMsg.applicationIdData = remoteFilename;
            remoteDirectory += "                                ";  // pad it with blanks.
            for (int i = 0; (i < remoteDirectory.length()) && (i < CMQC.MQ_ACCOUNTING_TOKEN_LENGTH); i++)
               sendMsg.accountingToken[i] = (byte) remoteDirectory.charAt(i);

            if (remoteJobFile != null)
            {
               sendMsg.feedback = sendMsg.feedback | CWConstants.UFM_START_JOB;
               if (mqso.remoteJobXmlFile.endsWith(CWConstants.XML_SUFFIX))
               { /* don't send the ".xml" part of the filename */
                  int xx = remoteJobFile.lastIndexOf(CWConstants.XML_SUFFIX);
                  remoteJobFile = remoteJobFile.substring(0,xx);
               }

               for (int i = 0; (i < remoteJobFile.length()) && (i < CMQC.MQ_CORREL_ID_LENGTH); i++)
                  sendMsg.correlationId[i] = (byte) remoteJobFile.charAt(i);
            }
            flag = true;
         }
      }
      
      return flag;
   }


   /**
    * Is setting of the MQMD Context required?
    *
    * @param mqso
    * @return flag
    */
   private boolean isSetContextRequired(MQSendObject mqso)
   {
      boolean flag = false;
      String remoteJobFile = null;
      String remoteFilename = null;
      String remoteDirectory = null;
      UFM.logger.debug("isSetContextRequired()");

      if (mqso.remoteJobXmlFile != null)
         remoteJobFile = Tokens.replace(mqso.actionName, mqso.remoteJobXmlFile, mqso.remoteJobXmlFile);

      remoteFilename = Tokens.replace(mqso.actionName, mqso.remoteFilename, mqso.srcFilename);

      remoteDirectory = Tokens.replace(mqso.actionName,
                                       mqso.remoteDirectory,
                                       IO.getPath(mqso.srcFilename) );

      if (mqso.usev7prop)
      {
         flag = false;
      }
      else
      {
         /**
          * Set MQMD Context fields if the data fits or
          * create XML and put the XML in the payload.
          */
         if ( (remoteFilename.length() > CMQC.MQ_APPL_IDENTITY_DATA_LENGTH)   ||
              (remoteDirectory.length() > CMQC.MQ_ACCOUNTING_TOKEN_LENGTH)    ||
              ( (remoteJobFile != null) &&
                ((remoteJobFile.length() - CWConstants.XML_SUFFIX.length()) > CMQC.MQ_CORREL_ID_LENGTH) ) )
         {
            flag = false;
         }
         else
         {
            flag = true;
         }
      }

      return flag;
   }

   /**
    * Handle the post common code.
    * @param mqso
    * @throws JDOMException
    * @throws IOException
    * @throws UFMException
    */
   private void handlePostCommon(MQSendObject mqso, InternalMessageQueue iq) throws JDOMException, IOException, UFMException
   {
      if (mqso.jobXmlFile != null)
      {
         RunJob rj = new RunJob();
         rj.launch(mqso.jobXmlFile, mqso.paClass.actionProperties, mqso.actionName, mqso.labelName);
      }

      if (mqso.archiveFile)
      {
         String archiveDirectory = Tokens.replace(mqso.archiveDirectory, mqso.paClass.actionProperties);
         String archiveFileName = null;
         if (mqso.archiveFileName != null)
            archiveFileName = Tokens.replace(mqso.actionName, mqso.archiveFileName, IO.getFileName(mqso.srcFilename) );
         else
            archiveFileName = IO.getFileName(mqso.srcFilename);

         handleDirectoryCheck(mqso.archiveCreateDir, archiveDirectory);
         
         try 
         {
            IO.archiveFile(mqso.srcFilename, archiveDirectory, archiveFileName);
         }
         catch (UFMException e)
         {
            if (mqso.onErrorFail)
               throw e;
            else
               UFM.handleOnError(mqso.paClass.actionProperties, iq);
         }
      }
      else if (mqso.deleteFile)
      {
         try 
         {
            IO.deleteFile(mqso.srcFilename);
         }
         catch (UFMException e)
         {
            if (mqso.onErrorFail)
               throw e;
            else
               UFM.handleOnError(mqso.paClass.actionProperties, iq);
         }
      }
      
      if ( (mqso.archiveFile) || (mqso.deleteFile) )
         mqso.paClass.handleDeletedWatchFoundFile();
   }
   
   /**
    * Create the archive directory.
    * @param archiveCreateDir
    * @param archiveDirectory
    * @throws IOException
    */
   private void handleDirectoryCheck(boolean archiveCreateDir, String archiveDirectory) throws IOException
   {
      /* if it does not exist then create it */
      if (!(new File(archiveDirectory).exists()))
      {
         if (archiveCreateDir)
         {
            if ((new File(archiveDirectory)).mkdirs()) 
            {
               UFM.logger.info("Successfully created archive directory: " + archiveDirectory);
            }
            else
            {
               String errorMsg = "Cannot create archive directory: " + archiveDirectory;
               UFM.logger.fatal(errorMsg);
               UFM.globalErrorText = errorMsg;
               throw new IOException("createArchiveDirectory: "+errorMsg);
            }
         }
      
      }
   }
}
