package com.capitalware.ufm.xml;

import java.io.*;
import java.util.ArrayList;
import java.util.StringTokenizer;

import org.jdom.*;
import org.jdom.input.*;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.xml.DTDs.ResolveLocalDTD;

/**
 * This class handles the reading and parsing of Email XML file and parse parameters
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class EmailXml extends UFMXmlAbstract
{
   private String    _smtpHost = null;
   private String    _smtpPort = "25";    // 465 or 587 is used for SSL
   private String    _smtpUserID = null;
   private String    _smtpPassword = null;
   private String    _fromEmailAddress = null;
   private String    _toEmailAddress = null;
   private String    _subject = null;
   private String    _messageText = null;
   private boolean   _ssl = false;
   private boolean   _auth = false;

   /**
    * The constructor
    * @param logger
    */
   public EmailXml()
   {
      super();
   }

   /**
    * Read Email XML file and set parameters
    *
    * @param xmlFilename
    *           Filename of job.xml file
    * @throws IOException
    */
   public void readXMLFile(String xmlFileName) throws JDOMException, IOException
   {
      SAXBuilder builder = new SAXBuilder(true);
      // handle the ability to pull DTD from Jar if needed
      builder.setEntityResolver(new ResolveLocalDTD(CWConstants.EMAIL_DTD));
      Document doc = builder.build(CWConstants.EMAIL_PATH + xmlFileName);
      Element rootElement = doc.getRootElement();
      Element smtpHostElement;

      if ((smtpHostElement = rootElement.getChild(CWConstants.HOSTNAME_ELEMENT)) != null)
      {
         _smtpHost = smtpHostElement.getTextTrim();
       
         if (smtpHostElement.getAttribute(CWConstants.SSL_ATTRIBUTE) != null)
         {
             char ssl = smtpHostElement.getAttribute(CWConstants.SSL_ATTRIBUTE).getValue().trim().charAt(0);
             if ( (ssl == 'Y') || (ssl == 'y') )
                _ssl = true;
             else
                _ssl = false;
         }
         
         if (smtpHostElement.getAttribute(CWConstants.AUTHENTICATION_ATTRIBUTE) != null)
         {
             char auth = smtpHostElement.getAttribute(CWConstants.AUTHENTICATION_ATTRIBUTE).getValue().trim().charAt(0);
             if ( (auth == 'Y') || (auth == 'y') )
                _auth = true;
             else
                _auth = false;
         }
      }

      if ( rootElement.getChild(CWConstants.PORT_ELEMENT) != null)
         _smtpPort = rootElement.getChild(CWConstants.PORT_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.USERID_ELEMENT) != null)
         _smtpUserID = rootElement.getChild(CWConstants.USERID_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.PASSWORD_ELEMENT) != null)
         _smtpPassword = rootElement.getChild(CWConstants.PASSWORD_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.FROM_EMAIL_ELEMENT) != null)
         _fromEmailAddress = rootElement.getChild(CWConstants.FROM_EMAIL_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.TO_EMAIL_ELEMENT) != null)
         _toEmailAddress = rootElement.getChild(CWConstants.TO_EMAIL_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.EMAIL_SUBJECT_ELEMENT) != null)
         _subject = rootElement.getChild(CWConstants.EMAIL_SUBJECT_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.EMAIL_MSG_TEXT_ELEMENT) != null)
         _messageText = rootElement.getChild(CWConstants.EMAIL_MSG_TEXT_ELEMENT).getTextTrim();
      
      UFM.logger.info("Values used for this email");
      UFM.logger.info("     SMTP Host            = " + _smtpHost);
      UFM.logger.info("     SMTP Port            = " + _smtpPort);
      UFM.logger.info("     SMTP SSL             = " + (_ssl?"Y":"N"));
      UFM.logger.info("     SMTP Authentication  = " + (_auth?"Y":"N"));
      UFM.logger.info("     SMTP UserID          = " + _smtpUserID);
      UFM.logger.info("     SMTP Password Length = " + (_smtpPassword == null ? _smtpPassword:_smtpPassword.length()));
      UFM.logger.info("     From Email Address   = " + _fromEmailAddress);
      UFM.logger.info("     To Email Address     = " + _toEmailAddress);
      UFM.logger.info("     Subject              = " + _subject);
      UFM.logger.info("     Message Text         = " + _messageText);
      
      if (_smtpHost == null)
         throw new JDOMException("Error: The SMTP host name cannot be blank.");
      
      if (_auth)
      {
         if (_smtpUserID == null)
            throw new JDOMException("Error: The SMTP UserID cannot be blank.");
  
         if (_smtpPassword == null)
            throw new JDOMException("Error: The SMTP Password cannot be blank.");
      }
      
      if (_fromEmailAddress == null)
         throw new JDOMException("Error:  The From Email Address cannot be blank.");
      
      if (_toEmailAddress == null)
         throw new JDOMException("Error:  The To Email Address cannot be blank.");
      
      if (_subject == null)
         throw new JDOMException("Error:  The email Subject cannot be blank.");
      
      if (_messageText == null)
         throw new JDOMException("Error:  The email Message Text cannot be blank.");
   }

   /**
    * Retrieve the smtpHost.
    * @return smtpHost
    */
   public String getSmtpHost()
   {
      return _smtpHost;
   }

   /**
    * Retrieve the smtpPort number.
    * @return smtpPort
    */
   public String getSmtpPort()
   {
      return _smtpPort;
   }

   /**
    * Retrieve the SmtpUserID.
    * @return smtpUserID
    */
   public String getSmtpUserID()
   {
      return _smtpUserID;
   }

   /**
    * Retrieve the smtpPassword.
    * @return smtpPassword
    */
   public String getSmtpPassword()
   {
      return _smtpPassword;
   }

   /**
    * Retrieve the "From" email address.
    * @return fromEmailAddress
    */
   public String getFromEmailAddress()
   {
      return _fromEmailAddress;
   }

   /**
    * Retrieve the "To" email addresses.
    * @return toEmailAddress
    */
   public ArrayList<String> getToAddress()
   {
      ArrayList<String> al = new ArrayList<String>();

      if (_toEmailAddress != null)
      {
         StringTokenizer st = new StringTokenizer(_toEmailAddress,",");
   
         while (st.hasMoreTokens())
            al.add(st.nextToken().trim());
      }
      
      return al;
   }

   /**
    * Retrieve the subject.
    * @return subject
    */
   public String getSubject()
   {
      return _subject;
   }

   /**
    * Retrieve the messageText.
    * @return messageText
    */
   public String getMessageText()
   {
      return _messageText;
   }

   /**
    * Is ssl set to true or false?
    * @return ssl
    */
   public boolean isSsl()
   {
      return _ssl;
   }

   /**
    * Is auth set to true or false?
    * @return auth
    */
   public boolean isAuth()
   {
      return _auth;
   }
}
