package com.capitalware.ufm.xml;

import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jdom.*;
import org.jdom.input.*;

import com.capitalware.ufm.handlers.StringUtilities;
import com.capitalware.ufm.mq.objects.MQObject;
import com.capitalware.ufm.mq.objects.QueueObject;
import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.xml.DTDs.ResolveLocalDTD;

/**
 * This class handles the reading and parsing of MQ XML file and parse parameters
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class MQXml extends UFMXmlAbstract
{
   private MQObject                    _mqo = null;
   public String                       qMgrName = null;
   public QueueObject                  qObject = null;
   public String                       CCDTFile = null;

   /**
    * The constructor
    * @param logger
    */
   public MQXml(String qMgrName, QueueObject qObject, String CCDTFile)
   {
      super();
      this.qMgrName = qMgrName;
      this.qObject = qObject;
      this.CCDTFile = CCDTFile;
   }

   /**
    * Read MQ XML file and set parameters
    *
    * @param xmlFilename
    *           Filename of job.xml file
    * @throws IOException
    */
   public void readXMLFile(String xmlFileName) throws JDOMException, IOException
   {
      SAXBuilder builder = new SAXBuilder(true);
      // handle the ability to pull DTD from Jar if needed
      builder.setEntityResolver(new ResolveLocalDTD(CWConstants.MQ_DTD));
      Document doc = builder.build(CWConstants.MQ_PATH + xmlFileName);
      Element rootElement = doc.getRootElement();
      Element disListElement = null;
      Element queueElement = null;
      
      _mqo = new MQObject(qMgrName, qObject, xmlFileName, CCDTFile);

      if ( rootElement.getChild(CWConstants.QMGRNAME_ELEMENT) != null)
         _mqo.qMgrName = rootElement.getChild(CWConstants.QMGRNAME_ELEMENT).getTextTrim();

      /* Distribution list?  */
      if ( (disListElement = rootElement.getChild(CWConstants.DISTRIBUTIONLIST_ELEMENT)) != null)
      {
         QueueObject mqdlo = null;
         _mqo.disListOfQNames = new ArrayList<QueueObject>();

         /* We should have at least 2 Queue elements */
         List propertyList = disListElement.getChildren(CWConstants.QUEUENAME_ELEMENT);
         Iterator it = propertyList.iterator();
         while (it.hasNext())
         {
            queueElement = (Element) it.next();
            
            mqdlo = new QueueObject();
            
            mqdlo.qName = queueElement.getTextTrim();
            
            if (queueElement.getAttribute(CWConstants.QMGRNAME_ATTRIBUTE) != null)
               mqdlo.qMgrName = queueElement.getAttribute(CWConstants.QMGRNAME_ATTRIBUTE).getValue().trim();
            else
               mqdlo.qMgrName = null;
            
            _mqo.disListOfQNames.add(mqdlo);
         }
      }
      else
      {  /* Queue name */
         if ( (queueElement = rootElement.getChild(CWConstants.QUEUENAME_ELEMENT)) != null)
         {
            _mqo.qObject = new QueueObject();
            _mqo.qObject.qName = queueElement.getTextTrim();
            
            if (queueElement.getAttribute(CWConstants.QMGRNAME_ATTRIBUTE) != null)
               _mqo.qObject.qMgrName = queueElement.getAttribute(CWConstants.QMGRNAME_ATTRIBUTE).getValue().trim();
            else
               _mqo.qObject.qMgrName = null;
         }
      }

      if ( rootElement.getChild(CWConstants.CCDTFILE_ELEMENT) != null)
         _mqo.CCDTFile = rootElement.getChild(CWConstants.CCDTFILE_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.HOSTNAME_ELEMENT) != null)
         _mqo.hostname = rootElement.getChild(CWConstants.HOSTNAME_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.PORT_ELEMENT) != null)
      {
         String num = rootElement.getChild(CWConstants.PORT_ELEMENT).getTextTrim();
         _mqo.port = StringUtilities.stringToInt(num, 1414);
      }

      if ( rootElement.getChild(CWConstants.CHANNELNAME_ELEMENT) != null)
         _mqo.channelName = rootElement.getChild(CWConstants.CHANNELNAME_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.USERID_ELEMENT) != null)
         _mqo.userID = rootElement.getChild(CWConstants.USERID_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.PASSWORD_ELEMENT) != null)
         _mqo.password = rootElement.getChild(CWConstants.PASSWORD_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.SECURITYEXIT_ELEMENT) != null)
         _mqo.securityExit = rootElement.getChild(CWConstants.SECURITYEXIT_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.SECURITYEXITPATH_ELEMENT) != null)
         _mqo.securityExitPath = rootElement.getChild(CWConstants.SECURITYEXITPATH_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.SECURITYEXITDATA_ELEMENT) != null)
         _mqo.securityExitData = rootElement.getChild(CWConstants.SECURITYEXITDATA_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.CIPHERSUITENAME_ELEMENT) != null)
         _mqo.cipherSuiteName = rootElement.getChild(CWConstants.CIPHERSUITENAME_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.DISTINGUISHEDNAME_ELEMENT) != null)
         _mqo.distinguishedName = rootElement.getChild(CWConstants.DISTINGUISHEDNAME_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.TRUSTEDSTORE_ELEMENT) != null)
         _mqo.trustedStore = rootElement.getChild(CWConstants.TRUSTEDSTORE_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.TRUSTEDSTOREPASSWD_ELEMENT) != null)
         _mqo.trustedStorePasswd = rootElement.getChild(CWConstants.TRUSTEDSTOREPASSWD_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.KEYSTORE_ELEMENT) != null)
         _mqo.keyStore = rootElement.getChild(CWConstants.KEYSTORE_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.KEYSTOREPASSWD_ELEMENT) != null)
         _mqo.keyStorePasswd = rootElement.getChild(CWConstants.KEYSTOREPASSWD_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.LDAPSERVER_ELEMENT) != null)
         _mqo.ldapServer = rootElement.getChild(CWConstants.LDAPSERVER_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.LDAPSERVERPORT_ELEMENT) != null)
      {
         String num = rootElement.getChild(CWConstants.LDAPSERVERPORT_ELEMENT).getTextTrim();
         _mqo.ldapServerPort = StringUtilities.stringToInt(num, 389);
      }
   }

   /**
    * Retrieve the CCDT file.
    * @return fileCCDT
    */
   public MQObject getMQObject()
   {
      return _mqo;
   }
}
