package com.capitalware.ufm.xml;

import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;

import org.jdom.*;
import org.jdom.input.*;

import com.capitalware.ufm.scp.objects.*;
import com.capitalware.ufm.xml.DTDs.ResolveLocalDTD;
import com.capitalware.ufm.handlers.StringUtilities;
import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.objects.ConnectionObject;

/**
 * This class handles the reading and parsing of Scp XML file and parse parameters
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class ScpXml extends UFMXmlAbstract
{
   private ConnectionObject      _fco = null;
   private ArrayList<ScpCmdObject>  _cmdsAL = null;

   /**
    * The constructor
    * @param logger
    */
   public ScpXml()
   {
      super();
   }

   /**
    * Read SCP XML file and set parameters
    *
    * @param xmlFilename
    *           Filename of job.xml file
    * @throws IOException
    */
   public void readXMLFile(String xmlFileName) throws JDOMException, IOException
   {
      SAXBuilder builder = new SAXBuilder(true);
      // handle the ability to pull DTD from Jar if needed
      builder.setEntityResolver(new ResolveLocalDTD(CWConstants.SCP_DTD));
      Document doc = builder.build(CWConstants.SCP_PATH + xmlFileName);
      Element rootElement = doc.getRootElement();
      Element SCPHostElement;
      
      _fco = new ConnectionObject();
      _fco.port = CWConstants.SCP_DEFAULT_PORT;

      if ((SCPHostElement = rootElement.getChild(CWConstants.HOSTNAME_ELEMENT)) != null)
         _fco.hostname = SCPHostElement.getTextTrim();

      if ( rootElement.getChild(CWConstants.PORT_ELEMENT) != null)
      {
         String num = rootElement.getChild(CWConstants.PORT_ELEMENT).getTextTrim();
         _fco.port = StringUtilities.stringToInt(num, CWConstants.SCP_DEFAULT_PORT);
      }

      if ( rootElement.getChild(CWConstants.USERID_ELEMENT) != null)
         _fco.userID = rootElement.getChild(CWConstants.USERID_ELEMENT).getTextTrim();

      if ( rootElement.getChild(CWConstants.PASSWORD_ELEMENT) != null)
         _fco.password = rootElement.getChild(CWConstants.PASSWORD_ELEMENT).getTextTrim();

      Element cmdsElement = rootElement.getChild(CWConstants.SCP_CMDS_ELEMENT);
      
      _cmdsAL = extractScpCmds(cmdsElement);
   }

   /**
    *
    *
    * @param xmlFilename
    *           The name of xml file
    * @throws JDOMException, IOException
    */
   private ArrayList<ScpCmdObject> extractScpCmds(Element SCPCmdsElement) throws JDOMException
   {
      ArrayList<ScpCmdObject> al = new ArrayList<ScpCmdObject>();

      Iterator it = SCPCmdsElement.getChildren().iterator();

      while (it.hasNext())
      {
         Element cmdElement = (Element) it.next();
         String cmdTagName = cmdElement.getName();

         if (CWConstants.SCP_COPY_FROM_ELEMENT.equals(cmdTagName))
            al.add(retrieveCopyFrom(cmdElement));
         else if  (CWConstants.SCP_COPY_TO_ELEMENT.equals(cmdTagName))
            al.add(retrieveCopyTo( cmdElement));
         else
            throw new JDOMException("Unknown ScpCmds tag '"+cmdTagName+"'");
      }

      return al;
   }

   /**
    * Retrieve all XML values for SCP Get command.
    * @param cmdElement
    * @throws JDOMException
    */
   private ScpCmdObject retrieveCopyFrom(Element cmdElement)
   {
      Element fileElement = null;

      CopyFromObject cfo = new CopyFromObject();

      if ((fileElement =  cmdElement.getChild(CWConstants.FILE_ELEMENT)) != null)
      {
         String filename = fileElement.getTextTrim();
         if ( (filename != null) && (!("".equals(filename))) )         
               cfo.localFilename = filename;

         if (fileElement.getAttribute(CWConstants.DIR_ATTRIBUTE) != null)
            cfo.localDir = fileElement.getAttribute(CWConstants.DIR_ATTRIBUTE).getValue().trim();
      }

      if (cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE) != null)
         cfo.remoteDir = cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE).getValue().trim();

      if (cmdElement.getAttribute(CWConstants.REMOTE_FILE_ATTRIBUTE) != null)
         cfo.remoteFilename = cmdElement.getAttribute(CWConstants.REMOTE_FILE_ATTRIBUTE).getValue().trim();

      return cfo;
   }

   /**
    * Retrieve all XML values for SCP Put command.
    * @param cmdElement
    * @throws JDOMException
    */
   private ScpCmdObject retrieveCopyTo(Element cmdElement)
   {
      Element fileElement = null;

      CopyToObject cto = new CopyToObject();
      
      if ((fileElement =  cmdElement.getChild(CWConstants.FILE_ELEMENT)) != null)
      {
         String filename = fileElement.getTextTrim();
         if ( (filename != null) && (!("".equals(filename))) )         
               cto.localFilename = filename;
         
         if (fileElement.getAttribute(CWConstants.DIR_ATTRIBUTE) != null)
            cto.localDir = fileElement.getAttribute(CWConstants.DIR_ATTRIBUTE).getValue().trim();
      }

      if (cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE) != null)
         cto.remoteDir = cmdElement.getAttribute(CWConstants.REMOTE_DIR_ATTRIBUTE).getValue().trim();

      if (cmdElement.getAttribute(CWConstants.REMOTE_FILE_ATTRIBUTE) != null)
         cto.remoteFilename = cmdElement.getAttribute(CWConstants.REMOTE_FILE_ATTRIBUTE).getValue().trim();

      return cto;
   }

   /**
    * Retrieve the ScpConnectionObject.
    * @return fco
    */
   public ConnectionObject getScpConnection()
   {
      return _fco;
   }

   /**
    * Retrieve the ScpCmdObject.
    * @return cmdsAL
    */
   public ArrayList<ScpCmdObject> getScpCmds()
   {
      return _cmdsAL;
   }
}
